"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.addBabelPlugin = addBabelPlugin;
exports.addBabelPreset = addBabelPreset;
exports.babelProjectPlugin = void 0;

var _tapable = require("tapable");

var _plugins = require("@sewing-kit/plugins");

const PLUGIN = 'SewingKit.babel';
const addBabelHooks = (0, _plugins.addHooks)(() => ({
  babelConfig: new _tapable.AsyncSeriesWaterfallHook(['babelConfig']),
  babelIgnorePatterns: new _tapable.AsyncSeriesWaterfallHook(['babelIgnorePatterns'])
}));
const addPackageBabelHooks = (0, _plugins.addHooks)(() => ({
  babelConfig: new _tapable.AsyncSeriesWaterfallHook(['babelConfig']),
  babelIgnorePatterns: new _tapable.AsyncSeriesWaterfallHook(['babelIgnorePatterns']),
  babelExtensions: new _tapable.AsyncSeriesWaterfallHook(['extensions'])
}));
const babelProjectPlugin = (0, _plugins.createProjectPlugin)({
  id: PLUGIN,

  run({
    build,
    test,
    dev
  }) {
    build.tap(PLUGIN, ({
      hooks
    }) => {
      hooks.package.tap(PLUGIN, ({
        hooks
      }) => {
        hooks.configure.tap(PLUGIN, addPackageBabelHooks);
      });
      hooks.webApp.tap(PLUGIN, ({
        hooks
      }) => {
        hooks.configure.tap(PLUGIN, addBabelHooks);
      });
      hooks.service.tap(PLUGIN, ({
        hooks
      }) => {
        hooks.configure.tap(PLUGIN, addBabelHooks);
      });
    });
    dev.tap(PLUGIN, ({
      hooks
    }) => {
      hooks.package.tap(PLUGIN, ({
        hooks
      }) => {
        hooks.configure.tap(PLUGIN, addBabelHooks);
      });
      hooks.webApp.tap(PLUGIN, ({
        hooks
      }) => {
        hooks.configure.tap(PLUGIN, addBabelHooks);
      });
      hooks.service.tap(PLUGIN, ({
        hooks
      }) => {
        hooks.configure.tap(PLUGIN, addBabelHooks);
      });
    });
    test.tap(PLUGIN, ({
      hooks
    }) => {
      hooks.project.tap(PLUGIN, ({
        hooks
      }) => {
        hooks.configure.tap(PLUGIN, addBabelHooks);
      });
    });
  }

});
exports.babelProjectPlugin = babelProjectPlugin;

function addBabelPlugin(plugin) {
  const id = `${PLUGIN}.AddBabelPlugin`;
  return createProjectPluginTargettingAllConfigurationHooks(id, hooks => {
    var _hooks$babelConfig;

    (_hooks$babelConfig = hooks.babelConfig) === null || _hooks$babelConfig === void 0 ? void 0 : _hooks$babelConfig.tap(id, config => {
      var _config$plugins;

      return { ...config,
        plugins: [...((_config$plugins = config.plugins) !== null && _config$plugins !== void 0 ? _config$plugins : []), plugin]
      };
    });
  });
}

function addBabelPreset(preset) {
  const id = `${PLUGIN}.AddBabelPreset`;
  return createProjectPluginTargettingAllConfigurationHooks(id, hooks => {
    var _hooks$babelConfig2;

    (_hooks$babelConfig2 = hooks.babelConfig) === null || _hooks$babelConfig2 === void 0 ? void 0 : _hooks$babelConfig2.tap(id, config => {
      var _config$presets;

      return { ...config,
        presets: [...((_config$presets = config.presets) !== null && _config$presets !== void 0 ? _config$presets : []), preset]
      };
    });
  });
}

function createProjectPluginTargettingAllConfigurationHooks(id, configurator) {
  return (0, _plugins.createProjectPlugin)({
    id,

    run({
      build,
      test,
      dev
    }) {
      build.tap(id, ({
        hooks
      }) => {
        hooks.package.tap(id, ({
          hooks
        }) => {
          hooks.configure.tap(id, configurator);
        });
        hooks.webApp.tap(id, ({
          hooks
        }) => {
          hooks.configure.tap(id, configurator);
        });
        hooks.service.tap(id, ({
          hooks
        }) => {
          hooks.configure.tap(id, configurator);
        });
      });
      dev.tap(id, ({
        hooks
      }) => {
        hooks.package.tap(id, ({
          hooks
        }) => {
          hooks.configure.tap(id, configurator);
        });
        hooks.webApp.tap(id, ({
          hooks
        }) => {
          hooks.configure.tap(id, configurator);
        });
        hooks.service.tap(id, ({
          hooks
        }) => {
          hooks.configure.tap(id, configurator);
        });
      });
      test.tap(id, ({
        hooks
      }) => {
        hooks.project.tap(id, ({
          hooks
        }) => {
          hooks.configure.tap(id, configurator);
        });
      });
    }

  });
}