"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tapable_1 = require("tapable");
const plugins_1 = require("@sewing-kit/plugins");
const PLUGIN = 'SewingKit.babel';
const addBabelHooks = plugins_1.addHooks(() => ({
    babelConfig: new tapable_1.AsyncSeriesWaterfallHook(['babelConfig']),
    babelIgnorePatterns: new tapable_1.AsyncSeriesWaterfallHook([
        'babelIgnorePatterns',
    ]),
}));
const addPackageBabelHooks = plugins_1.addHooks(() => ({
    babelConfig: new tapable_1.AsyncSeriesWaterfallHook(['babelConfig']),
    babelIgnorePatterns: new tapable_1.AsyncSeriesWaterfallHook([
        'babelIgnorePatterns',
    ]),
    babelExtensions: new tapable_1.AsyncSeriesWaterfallHook([
        'extensions',
    ]),
}));
exports.babelProjectPlugin = plugins_1.createProjectPlugin({
    id: PLUGIN,
    run({ build, test, dev }) {
        build.tap(PLUGIN, ({ hooks }) => {
            hooks.package.tap(PLUGIN, ({ hooks }) => {
                hooks.configure.tap(PLUGIN, addPackageBabelHooks);
            });
            hooks.webApp.tap(PLUGIN, ({ hooks }) => {
                hooks.configure.tap(PLUGIN, addBabelHooks);
            });
            hooks.service.tap(PLUGIN, ({ hooks }) => {
                hooks.configure.tap(PLUGIN, addBabelHooks);
            });
        });
        dev.tap(PLUGIN, ({ hooks }) => {
            hooks.package.tap(PLUGIN, ({ hooks }) => {
                hooks.configure.tap(PLUGIN, addBabelHooks);
            });
            hooks.webApp.tap(PLUGIN, ({ hooks }) => {
                hooks.configure.tap(PLUGIN, addBabelHooks);
            });
            hooks.service.tap(PLUGIN, ({ hooks }) => {
                hooks.configure.tap(PLUGIN, addBabelHooks);
            });
        });
        test.tap(PLUGIN, ({ hooks }) => {
            hooks.project.tap(PLUGIN, ({ hooks }) => {
                hooks.configure.tap(PLUGIN, addBabelHooks);
            });
        });
    },
});
function addBabelPlugin(plugin) {
    const id = `${PLUGIN}.AddBabelPlugin`;
    return createProjectPluginTargettingAllConfigurationHooks(id, (hooks) => {
        var _a;
        (_a = hooks.babelConfig) === null || _a === void 0 ? void 0 : _a.tap(id, (config) => {
            var _a;
            return (Object.assign(Object.assign({}, config), { plugins: [...(_a = config.plugins, (_a !== null && _a !== void 0 ? _a : [])), plugin] }));
        });
    });
}
exports.addBabelPlugin = addBabelPlugin;
function addBabelPreset(preset) {
    const id = `${PLUGIN}.AddBabelPreset`;
    return createProjectPluginTargettingAllConfigurationHooks(id, (hooks) => {
        var _a;
        (_a = hooks.babelConfig) === null || _a === void 0 ? void 0 : _a.tap(id, (config) => {
            var _a;
            return (Object.assign(Object.assign({}, config), { presets: [...(_a = config.presets, (_a !== null && _a !== void 0 ? _a : [])), preset] }));
        });
    });
}
exports.addBabelPreset = addBabelPreset;
function createProjectPluginTargettingAllConfigurationHooks(id, configurator) {
    return plugins_1.createProjectPlugin({
        id,
        run({ build, test, dev }) {
            build.tap(id, ({ hooks }) => {
                hooks.package.tap(id, ({ hooks }) => {
                    hooks.configure.tap(id, configurator);
                });
                hooks.webApp.tap(id, ({ hooks }) => {
                    hooks.configure.tap(id, configurator);
                });
                hooks.service.tap(id, ({ hooks }) => {
                    hooks.configure.tap(id, configurator);
                });
            });
            dev.tap(id, ({ hooks }) => {
                hooks.package.tap(id, ({ hooks }) => {
                    hooks.configure.tap(id, configurator);
                });
                hooks.webApp.tap(id, ({ hooks }) => {
                    hooks.configure.tap(id, configurator);
                });
                hooks.service.tap(id, ({ hooks }) => {
                    hooks.configure.tap(id, configurator);
                });
            });
            test.tap(id, ({ hooks }) => {
                hooks.project.tap(id, ({ hooks }) => {
                    hooks.configure.tap(id, configurator);
                });
            });
        },
    });
}
